import 'package:flutter/material.dart';
import 'package:sw_component/components/widget/date_picker.dart';
import 'package:sw_component/components/widget/error_box.dart';
import 'package:sw_component/components/widget/image_compat.dart';
import 'package:sw_component/components/widget/item_form_field.dart';
import 'package:sw_component/components/widget/material_theme_wrapper.dart';
import 'package:sw_component/components/widget/result_screen.dart';
import 'package:sw_component/components/widget/slim_appbar.dart';
import 'package:sw_component/tool/format.dart';
import 'package:sw_core/entity_auth/auth_step_params.dart';
import 'package:sw_core/entity_auth/communication_details.dart';
import 'package:sw_core/entity_auth/login_security_details.dart';
import 'package:sw_core/entity_auth/password_step.dart';
import 'package:sw_core/entity_auth/security_details.dart';
import 'package:sw_core/entity_auth/user_pass.dart';
import 'package:sw_core/entity_theme/params_loan_connect.dart';
import 'package:sw_core/interface/controller/i_controller_auth.dart';
import 'package:sw_core/request/request_auth.dart';
import 'package:sw_core/request/request_auth_add_steps.dart';
import 'package:sw_core/tool/inject.dart';
import 'package:sw_core/navigation/nav.dart';
import 'package:sw_core/tool/observer.dart';
import 'package:sw_core/tool/state.dart';

import '../input/event_handler_loan_connect.dart';

routeUnintegrated(BuildContext? context) => navigate(context, UnintegratedLoginScreen());

class UnintegratedLoginScreen extends StatelessWidget {
  UnintegratedLoginScreen({Key? key}) : super(key: key);

  final ParamsLoanConnect params = get<ParamsLoanConnect>();

  final RawBlueItemFormField _firstName = _newField("First name");
  final RawBlueItemFormField _lastName = _newField("Last name");
  final RawBlueItemFormField _emailID = _newField("Email ID");
  final RawBlueItemFormField _loanServicerName = _newField("Loan servicer name *");
  final RawBlueItemFormField _username = _newField("Username *");
  final RawBlueItemFormField _password = _newField("Password *", obscureText: true);
  final RawBlueItemFormField _pin = _newField("Pin");
  final BlueDatePicker _datePicker = BlueDatePicker(
    showLabel: true,
    label: "Date of birth",
  );
  final RawBlueItemFormField _accountNumber = _newField("Account number");

  final _isLoading = false.obs;
  final _errorBoxVisibility = false.obs;
  final _controllerAuth = getSafe<IControllerAuth>();
  final ScrollController _scrollController = ScrollController();
  BuildContext? context;

  @override
  Widget build(BuildContext context) {
    this.context = context;
    return MaterialThemeWrapper(
        child: Scaffold(
      backgroundColor: colorScheme.background,
      body: Observer(() => _checkAuthStep()),
    ));
  }

  _checkAuthStep() {
    switch (_controllerAuth!.authStep.value) {
      case AuthStep.STEP_UNAUTHORIZED:
        return _initialState(false);
      case AuthStep.STEP_SECURITY:
        return _authStepSecurity();
      case AuthStep.STEP_AUTHORIZED:
        return _authCompleted();
      case AuthStep.AUTH_ERROR:
        return _authError();
    }
  }

  Widget title() => Container(
        margin: EdgeInsets.only(top: 16, bottom: 20, left: 16),
        child: Text(
          'Enter your loan information',
          style: theme.textTheme.headline4,
        ),
      );

  List<Widget> _formBody() => [
        _firstName,
        _lastName,
        _emailID,
        _loanServicerName,
        _username,
        _password,
        _pin,
        _datePicker,
        _accountNumber
      ];

  Widget _initialState(bool hasError) => SingleChildScrollView(
        controller: _scrollController,
        child: Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          mainAxisAlignment: MainAxisAlignment.center,
          children: [
            SlimAppBar(
              '',
              onBackPressed: _onBackPressed,
            ),
            title(),
            hasError ? _createErrorBox() : SizedBox(),
            ..._formBody(),
            SizedBox(height: 56),
            _submitButton()
          ],
        ),
      );

  _authStepSecurity() {
    _controllerAuth!.authStep.value = AuthStep.STEP_SECURITY;
    final SecurityStep security = _controllerAuth!.security.value;

    List<BlueItemFormField> formItems = security.securityDetails
        .asMap()
        .entries
        .map(
          (entry) => BlueItemFormField(
            label: entry.value.question,
            initialValue: entry.value.answer,
          ),
        )
        .toList();

    return Center(
      child: Scaffold(
        backgroundColor: colorScheme.background,
        appBar: SlimAppBar(
          '',
          onBackPressed: _onBackPressed,
        ),
        bottomNavigationBar: _continueButtonSecurityQuestions(security, formItems),
        body: Center(
          child: SingleChildScrollView(
            controller: _scrollController,
            child: Column(
              mainAxisAlignment: MainAxisAlignment.spaceEvenly,
              children: [
                ...formItems,
                const SizedBox(height: 16),
              ],
            ),
          ),
        ),
      ),
    );
  }

  Widget _continueButtonSecurityQuestions(
          SecurityStep security, List<BlueItemFormField> formItems) =>
      IgnorePointer(
        ignoring: _isLoading.value,
        child: Container(
          margin: EdgeInsets.only(left: 30, right: 30, bottom: 16),
          height: 48,
          child: ElevatedButton(
            style: ElevatedButton.styleFrom(
              splashFactory: NoSplash.splashFactory,
              onPrimary: colorScheme.secondary,
              onSurface: colorScheme.secondary,
            ),
            onPressed: () async {
              await _requestAuthAddSteps(
                  security,
                  AuthStepParams(accountUserName: security.nextAuthStepParams.accountUserName),
                  LoginSecurityDetails(
                    securityDetails: List.generate(
                      security.securityDetails.length,
                      (index) => SecurityDetails(
                        questionId: security.securityDetails[index].questionId,
                        question: security.securityDetails[index].question,
                        answer: formItems[index].field.controller?.text,
                      ),
                    ),
                  ));
              _isLoading.value = true;
            },
            child: Observer(() => _isLoading.value
                ? _loading()
                : Text(
                    'Continue',
                    style: TextStyle(
                      color: colorScheme.onPrimary,
                    ),
                  )),
          ),
        ),
      );

  Widget _authError() {
    setVisibility(true);
    _scrollToTop();
    return _initialState(true);
  }

  _authCompleted() => Scaffold(
      backgroundColor: colorScheme.background,
      appBar: SlimAppBar(
        '',
        onBackPressed: _onBackPressed,
      ),
      body: ResultScreen(
          hasExitHeader: false,
          icon: params.servicerNotFoundIcon,
          title: 'We can’t find your servicer',
          bodyText:
              'Looks like we couldn’t find your loan servicer. You can try again or continue without connecting.',
          firstButtonPressed: _onBackPressed,
          secondButtonPressed: getSafe<EventHandlerLoanConnect>()?.onContinue ??
              () {
                if (context != null) {
                  ScaffoldMessenger.of(context!).showSnackBar(
                    SnackBar(
                      behavior: SnackBarBehavior.floating,
                      content: Text("Replace me with a navigation callback!"),
                    ),
                  );
                }
              },
          firstButtonText: 'Connect a different loan',
          secondButtonText: 'Continue without connecting'));

  Widget _submitButton() => Container(
        height: 48,
        width: double.infinity,
        margin: EdgeInsets.only(left: 16, right: 16, bottom: 24),
        child: ElevatedButton(
          style: ElevatedButton.styleFrom(splashFactory: NoSplash.splashFactory),
          child: Observer(
              () => _isLoading.value ? _loading() : Text('Submit', style: TextStyle(fontSize: 16))),
          onPressed: _sendRequestAuth,
        ),
      );

  _scrollToTop() => _scrollController.animateTo(0,
      duration: Duration(milliseconds: 500), curve: Curves.fastOutSlowIn);

  Widget _loading() => CircularProgressIndicator(
        valueColor: AlwaysStoppedAnimation<Color>(Colors.white),
      );

  Widget _createErrorBox({String? message, EdgeInsets? margin}) => Center(
        child: Container(
          margin: margin ?? EdgeInsets.only(left: 16, right: 16, top: 20),
          child: Observer(
            () => _errorBoxVisibility.value
                ? ErrorBox(
                    _reset,
                    message: message ??
                        "You must fill at least the loan servicer name, username and password. Kindly recheck.",
                    width: double.infinity,
                  )
                : Container(),
          ),
        ),
      );

  Future _sendRequestAuth() async {
    _isLoading.value = true;
    await _controllerAuth?.requestAuth(RequestAuth(
        email: CommunicationDetails(email: _emailID.controller!.text),
        loginDetails: UserPass(
            firstName: _firstName.controller!.text,
            lastName: _lastName.controller!.text,
            emailID: _emailID.controller!.text,
            loanServicerName: _loanServicerName.controller!.text,
            username: _username.controller!.text,
            password: _password.controller!.text,
            pin: _pin.controller!.text,
            dob: _datePicker.selectedDate == formatNumbersUSA(DateTime.now().toString())
                ? null
                : _datePicker.selectedDate,
            accountNumber: _accountNumber.controller!.text),
        loanServicerName: _loanServicerName.controller!.text,
        time: DateTime.now().millisecondsSinceEpoch,
        liabilityType: 'STUDENT_LOAN'));
    _isLoading.value = false;
  }

  Future _requestAuthAddSteps(
    SecurityStep security,
    AuthStepParams authStepParams,
    LoginSecurityDetails loginSecurityDetails,
  ) async {
    _isLoading.value = true;
    await _controllerAuth!.requestAuthAddSteps(RequestAuthAddSteps(
        loanServicerName: _loanServicerName.controller!.text,
        userId: security.userId,
        loginDetails: loginSecurityDetails,
        authStep: security.nextAuthStep,
        authStepParams: authStepParams,
        time: DateTime.now().millisecondsSinceEpoch,
        liabilityType: 'STUDENT_LOAN'));
    _isLoading.value = false;
  }

  void _reset() {
    setVisibility(false);
    _controllerAuth!.authStep.value = AuthStep.STEP_UNAUTHORIZED;
  }

  void _onBackPressed() {
    _controllerAuth!.authStep.value = AuthStep.STEP_UNAUTHORIZED;
    _controllerAuth!.security.value = SecurityStep();
    back(context);
  }

  void setVisibility(bool value) => _errorBoxVisibility.value = value;
}

_newField(String label, {bool? obscureText}) => RawBlueItemFormField(
      obscureText: obscureText ?? false,
      controller: TextEditingController(),
      label: label,
      showHint: false,
    );
